@@ -13,6 +13,7 @@ import {
     TupleStorageFactoryService,
     VortexService,
     VortexStatusService,
+    TupleStorageBatchSaveArguments,
 } from "@synerty/vortexjs";
 
 import {
@@ -28,7 +29,7 @@ import { DocDbDocumentTypeTuple } from "../../DocDbDocumentTypeTuple";
 import { PrivateDocumentLoaderStatusTuple } from "./PrivateDocumentLoaderStatusTuple";
 import { DocDbModelSetTuple } from "../../DocDbModelSetTuple";
 import {
-    DeviceOfflineCacheControllerService,
+    DeviceOfflineCacheService,
     OfflineCacheStatusTuple,
 } from "@peek/peek_core_device";
 
@@ -121,14 +122,19 @@ function keyChunk(modelSetKey: string, key: string): string {
 @Injectable()
 export class PrivateDocumentLoaderService extends NgLifeCycleEvents {
     private UPDATE_CHUNK_FETCH_SIZE = 5;
-    private OFFLINE_CHECK_PERIOD_MS = 15 * 60 * 1000; // 15 minutes
+
+    // Every 100 chunks from the server
+    private SAVE_POINT_ITERATIONS = 100;
+
+    // Saving the cache after each chunk is so expensive, we only do it every 20 or so
+    private chunksSavedSinceLastIndexSave = 0;
 
     private index = new DocumentUpdateDateTuple();
     private askServerChunks: DocumentUpdateDateTuple[] = [];
 
     private _hasLoaded = false;
-
     private _hasLoadedSubject = new Subject<void>();
+
     private storage: TupleOfflineStorageService;
 
     private _statusSubject = new Subject<PrivateDocumentLoaderStatusTuple>();
@@ -145,7 +151,7 @@ export class PrivateDocumentLoaderService extends NgLifeCycleEvents {
         private vortexStatusService: VortexStatusService,
         storageFactory: TupleStorageFactoryService,
         private tupleService: DocDbTupleService,
-        private deviceCacheControllerService: DeviceOfflineCacheControllerService
+        private deviceCacheControllerService: DeviceOfflineCacheService
     ) {
         super();
 
@@ -335,7 +341,7 @@ export class PrivateDocumentLoaderService extends NgLifeCycleEvents {
             )
             .pipe(takeUntil(this.onDestroyEvent))
             .subscribe((payloadEnvelope: PayloadEnvelope) => {
-                this.processDocumentsFromServer(payloadEnvelope);
+                this.processChunksFromServer(payloadEnvelope);
             });
 
         // If the vortex service comes back online, update the watch grids.
@@ -403,6 +409,7 @@ export class PrivateDocumentLoaderService extends NgLifeCycleEvents {
         if (!this.areWeTalkingToTheServer()) return;
 
         this.askServerChunks = [];
+        this.chunksSavedSinceLastIndexSave = 0;
 
         let count = 0;
         let indexChunk = new DocumentUpdateDateTuple();
@@ -431,93 +438,100 @@ export class PrivateDocumentLoaderService extends NgLifeCycleEvents {
 
         if (this.askServerChunks.length == 0) return;
 
-        let indexChunk: DocumentUpdateDateTuple = this.askServerChunks.pop();
-        let filt = extend({}, clientDocumentWatchUpdateFromDeviceFilt);
-        filt[cacheAll] = true;
-        let pl = new Payload(filt, [indexChunk]);
-        this.vortexService.sendPayload(pl);
+        this.deviceCacheControllerService //
+            .waitForGarbageCollector()
+            .then(() => {
+                let indexChunk: DocumentUpdateDateTuple =
+                    this.askServerChunks.pop();
+                let filt = extend({}, clientDocumentWatchUpdateFromDeviceFilt);
+                filt[cacheAll] = true;
+                let pl = new Payload(filt, [indexChunk]);
+                this.vortexService.sendPayload(pl);
 
-        this._status.lastCheck = new Date();
-        this._notifyStatus();
+                this._status.lastCheck = new Date();
+                this._notifyStatus();
+            });
     }
 
     /** Process Documentes From Server
      *
      * Process the grids the server has sent us.
      */
-    private processDocumentsFromServer(payloadEnvelope: PayloadEnvelope) {
+    private async processChunksFromServer(payloadEnvelope: PayloadEnvelope) {
         if (payloadEnvelope.result != null && payloadEnvelope.result != true) {
             console.log(`ERROR: ${payloadEnvelope.result}`);
             return;
         }
 
-        payloadEnvelope
-            .decodePayload()
-            .then((payload: Payload) => this.storeDocumentPayload(payload))
-            .then(() => {
-                if (this.askServerChunks.length == 0) {
-                    this.index.initialLoadComplete = true;
-                    this._hasLoaded = true;
-                    this._hasLoadedSubject.next();
-                } else if (payloadEnvelope.filt[cacheAll] == true) {
-                    this.askServerForNextUpdateChunk();
-                }
-            })
-            .then(() => this._notifyStatus())
-            .catch(
-                (e) => `DocumentCache.processDocumentsFromServer failed: ${e}`
-            );
-    }
-
-    private storeDocumentPayload(payload: Payload) {
-        let tuplesToSave: EncodedDocumentChunkTuple[] = <
+        const tuplesToSave: EncodedDocumentChunkTuple[] = <
             EncodedDocumentChunkTuple[]
-        >payload.tuples;
-        if (tuplesToSave.length == 0) return;
+        >payloadEnvelope.data;
 
-        // 2) Store the index
-        this.storeDocumentChunkTuples(tuplesToSave)
-            .then(() => {
-                // 3) Store the update date
+        try {
+            await this.storeChunkTuples(tuplesToSave);
+        } catch (e) {
+            console.log(`DocumentCache.storeDocumentPayload: ${e}`);
+        }
 
-                for (let docDbIndex of tuplesToSave) {
-                    this.index.updateDateByChunkKey[docDbIndex.chunkKey] =
-                        docDbIndex.lastUpdate;
-                }
+        this.chunksSavedSinceLastIndexSave += tuplesToSave.length;
 
-                return this.storage.saveTuples(new UpdateDateTupleSelector(), [
-                    this.index,
-                ]);
-            })
-            .catch((e) =>
-                console.log(`DocumentCache.storeDocumentPayload: ${e}`)
-            );
+        if (this.askServerChunks.length == 0) {
+            this.index.initialLoadComplete = true;
+            await this.saveChunkCacheIndex(true);
+            this._hasLoaded = true;
+            this._hasLoadedSubject.next();
+        } else if (payloadEnvelope.filt[cacheAll] == true) {
+            this.askServerForNextUpdateChunk();
+        }
+
+        this._notifyStatus();
     }
 
     /** Store Index Bucket
      * Stores the index bucket in the local db.
      */
-    private storeDocumentChunkTuples(
-        encodedDocumentChunkTuples: EncodedDocumentChunkTuple[]
+    private async storeChunkTuples(
+        tuplesToSave: EncodedDocumentChunkTuple[]
     ): Promise<void> {
-        let retPromise: any;
-        retPromise = this.storage.transaction(true).then((tx) => {
-            let promises = [];
+        // noinspection BadExpressionStatementJS
+        const Selector = DocumentChunkTupleSelector;
 
-            for (let encodedDocumentChunkTuple of encodedDocumentChunkTuples) {
-                promises.push(
-                    tx.saveTuplesEncoded(
-                        new DocumentChunkTupleSelector(
-                            encodedDocumentChunkTuple.chunkKey
-                        ),
-                        encodedDocumentChunkTuple.encodedData
-                    )
-                );
-            }
+        if (tuplesToSave.length == 0) return;
 
-            return Promise.all(promises).then(() => tx.close());
-        });
-        return retPromise;
+        const batchStore: TupleStorageBatchSaveArguments[] = [];
+        for (const tuple of tuplesToSave) {
+            batchStore.push({
+                tupleSelector: new Selector(tuple.chunkKey),
+                vortexMsg: tuple.encodedData,
+            });
+        }
+
+        await this.storage.batchSaveTuplesEncoded(batchStore);
+
+        for (const tuple of tuplesToSave) {
+            this.index.updateDateByChunkKey[tuple.chunkKey] = tuple.lastUpdate;
+        }
+        await this.saveChunkCacheIndex();
+    }
+
+    /** Store Chunk Cache Index
+     *
+     * Updates our running tab of the update dates of the cached chunks
+     *
+     */
+    private async saveChunkCacheIndex(force = false): Promise<void> {
+        if (
+            this.chunksSavedSinceLastIndexSave <= this.SAVE_POINT_ITERATIONS &&
+            !force
+        ) {
+            return;
+        }
+
+        this.chunksSavedSinceLastIndexSave = 0;
+
+        await this.storage.saveTuples(new UpdateDateTupleSelector(), [
+            this.index,
+        ]);
     }
 
     /** Get Documents When Ready
